@@ -21,11 +21,11 @@ def _get_friends(molecule, atom1, atom2):
     if atom1.residue != atom2.residue:
         if molecule.atom_index[atom1.residue.atoms[0]] < molecule.atom_index[atom1.residue.atoms[1]]:
             res_index = molecule.atom_index[atom1.residue.atoms[-1]]
-            atom1_friends = list(range(res_index))
+            atom1_friends = list(range(res_index+1))
             atom2_friends = list(range(res_index+1, len(molecule.atoms)))
         else:
             res_index = molecule.atom_index[atom2.residue.atoms[-1]]
-            atom2_friends = list(range(res_index))
+            atom2_friends = list(range(res_index+1))
             atom1_friends = list(range(res_index+1, len(molecule.atoms)))
     else:
         link_front = 0
@@ -62,11 +62,9 @@ def _get_friends(molecule, atom1, atom2):
                 index_next.update(index_temp)
             index_dict = index_next
             
-       
         index_dict = {}.fromkeys(restype.connectivity[typeatom2], typeatom2)
         if typeatom1 in index_dict.keys():
             index_dict.pop(typeatom1)
-        
         while index_dict:
             index_next = {}
             for atom0, from_atom in index_dict.items():
@@ -170,35 +168,76 @@ def Impose_Dihedral(molecule, atom1, atom2, atom3, atom4, dihedral):
 
 sys.modules['__main__'].__dict__["Impose_Dihedral"] = Impose_Dihedral 
 
-def _link_residue_process_coordinate(molecule, atom1, atom2, atom10, atom100, linkbond, linkangle, linkdihedral):
+def _link_residue_process_coordinate(molecule, atom1, atom2):
+    resA = atom1.residue
+    resB = atom2.residue
     crd = _get_crd(molecule)
     atom1_friends, atom2_friends = _get_friends(molecule, atom1, atom2)
     crd[atom2_friends] += 2000
-    r0 = crd[molecule.atom_index[atom2]] - crd[molecule.atom_index[atom1]]
-    L0 = np.linalg.norm(r0)
-    dr = (linkbond/L0 - 1) * r0
-    crd[atom2_friends] += dr
     
-    if atom10:              
-        r101 = crd[molecule.atom_index[atom10]] - crd[molecule.atom_index[atom1]]
-        angle0 = np.arccos(np.dot(r0, r101)  / np.linalg.norm(r0) / np.linalg.norm(r101))
-        deltaAngle =  linkangle - angle0 
-        crd[atom2_friends] = np.dot(crd[atom2_friends] - crd[molecule.atom_index[atom1]], _get_rotate_matrix( np.cross(r101, r0), deltaAngle)) + crd[molecule.atom_index[atom1]]
+    res = resA
+    atomA = atom1
+    atomB = atom2
+    atomB_friends = atom2_friends
+    for link_conditions in res.type.tail_link_conditions:
+        atoms = [ res._name2atom[atom] for atom in link_conditions["atoms"]]
+        parameter = link_conditions["parameter"]
+        if len(atoms) == 1:
+            r0 = crd[molecule.atom_index[atomB]] - crd[molecule.atom_index[atoms[0]]]
+            L0 = np.linalg.norm(r0)
+            dr = (parameter/L0 - 1) * r0
+            crd[atomB_friends] += dr
+        elif len(atoms) == 2:
+            rAO =  crd[molecule.atom_index[atoms[0]]] - crd[molecule.atom_index[atoms[1]]]
+            rOB =  crd[molecule.atom_index[atomB]] - crd[molecule.atom_index[atoms[1]]]
+            angle0 = np.arccos(np.dot(rAO, rOB)  / np.linalg.norm(rAO) / np.linalg.norm(rOB))
+            deltaAngle =  parameter - angle0
+            crd[atomB_friends] = np.dot(crd[atomB_friends] - crd[molecule.atom_index[atoms[1]]], _get_rotate_matrix( np.cross(rAO, rOB), deltaAngle)) + crd[molecule.atom_index[atoms[1]]]        
         
-        if atom100:
-            atom1_friends, atom2_friends = _get_friends(molecule, atom10, atom1)
-            r000 = crd[molecule.atom_index[atom100]] - crd[molecule.atom_index[atom10]]
-            r101 = crd[molecule.atom_index[atom1]] - crd[molecule.atom_index[atom10]]
-            r0 = crd[molecule.atom_index[atom1]] - crd[molecule.atom_index[atom2]]
-            r12xr23 = np.cross(r000, r101)
-            r23xr34 = np.cross(r0, r101)
+        elif len(atoms) == 3:
+            rOO =  crd[molecule.atom_index[atoms[0]]] - crd[molecule.atom_index[atoms[1]]]
+            rOA =  crd[molecule.atom_index[atoms[1]]] - crd[molecule.atom_index[atoms[2]]]
+            rAB =  crd[molecule.atom_index[atoms[1]]] - crd[molecule.atom_index[atomB]]
+            r12xr23 = np.cross(rOO, rOA)
+            r23xr34 = np.cross(rAB, rOA)
             cos = np.dot(r12xr23, r23xr34)  / np.linalg.norm(r12xr23) / np.linalg.norm(r23xr34)
             cos = max(-0.999999, min(cos, 0.999999))
             dihedral0 = np.arccos(cos)
-            dihedral0 = np.pi - np.copysign(dihedral0, np.cross(r23xr34, r12xr23).dot(r101))
-            deltaAngle = linkdihedral - dihedral0
-            crd[atom2_friends] = np.dot(crd[atom2_friends] - crd[molecule.atom_index[atom1]], _get_rotate_matrix(r101, deltaAngle)) + crd[molecule.atom_index[atom1]]
-        
+            dihedral0 = np.pi - np.copysign(dihedral0, np.cross(r23xr34, r12xr23).dot(rOA))
+            deltaAngle = parameter - dihedral0
+            crd[atomB_friends] = np.dot(crd[atomB_friends] - crd[molecule.atom_index[atoms[2]]], _get_rotate_matrix(rOA, deltaAngle)) + crd[molecule.atom_index[atoms[2]]]
+
+    res = resB
+    atomA = atom2
+    atomB = atom1
+    atomB_friends = atom1_friends
+    for link_conditions in res.type.head_link_conditions:
+        atoms = [ res._name2atom[atom] for atom in link_conditions["atoms"]]
+        parameter = link_conditions["parameter"]
+        if len(atoms) == 1:
+            r0 = crd[molecule.atom_index[atomB]] - crd[molecule.atom_index[atoms[0]]]
+            L0 = np.linalg.norm(r0)
+            dr = (parameter/L0 - 1) * r0
+            crd[atomB_friends] += dr
+        elif len(atoms) == 2:
+            rAO =  crd[molecule.atom_index[atoms[0]]] - crd[molecule.atom_index[atoms[1]]]
+            rOB =  crd[molecule.atom_index[atomB]] - crd[molecule.atom_index[atoms[1]]]
+            angle0 = np.arccos(np.dot(rAO, rOB)  / np.linalg.norm(rAO) / np.linalg.norm(rOB))
+            deltaAngle =  parameter - angle0
+            crd[atomB_friends] = np.dot(crd[atomB_friends] - crd[molecule.atom_index[atoms[1]]], _get_rotate_matrix( np.cross(rAO, rOB), deltaAngle)) + crd[molecule.atom_index[atoms[1]]]        
+        elif len(atoms) == 3:
+            rOO =  crd[molecule.atom_index[atoms[0]]] - crd[molecule.atom_index[atoms[1]]]
+            rOA =  crd[molecule.atom_index[atoms[1]]] - crd[molecule.atom_index[atoms[2]]]
+            rAB =  crd[molecule.atom_index[atoms[1]]] - crd[molecule.atom_index[atomB]]
+            r12xr23 = np.cross(rOO, rOA)
+            r23xr34 = np.cross(rAB, rOA)
+            cos = np.dot(r12xr23, r23xr34)  / np.linalg.norm(r12xr23) / np.linalg.norm(r23xr34)
+            cos = max(-0.999999, min(cos, 0.999999))
+            dihedral0 = np.arccos(cos)
+            dihedral0 = np.pi - np.copysign(dihedral0, np.cross(r23xr34, r12xr23).dot(rOA))
+            deltaAngle = parameter - dihedral0
+            crd[atomB_friends] = np.dot(crd[atomB_friends] - crd[molecule.atom_index[atoms[2]]], _get_rotate_matrix(rOA, deltaAngle)) + crd[molecule.atom_index[atoms[2]]]
+    
     for atom in molecule.atoms:
         i = molecule.atom_index[atom]
         atom.x = crd[i][0]
@@ -216,20 +255,11 @@ def ResidueType_Add(self, other):
             resB.Add_Atom(atom)
         new_molecule.Add_Residue(resA)
         new_molecule.Add_Residue(resB)
-        assert resA.type.tail and resB.type.head, "+ and * can only be used to link head and tail"
-        atom1 = resA._name2atom[self.tail]
-        atom2 = resB._name2atom[other.head] 
-        new_molecule.Add_Residue_Link(atom1, atom2)
-        if resA.type.tail_second:
-            atom10 = resA._name2atom[resA.type.tail_second]
-        else:
-            atom10 = None
-        if resA.type.tail_third:
-            atom100 = resA._name2atom[resA.type.tail_third]
-        else:
-            atom100 = None
-        _link_residue_process_coordinate(new_molecule, atom1, atom2, atom10, atom100, 
-            resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)            
+        if resA.type.tail and resB.type.head:
+            atom1 = resA._name2atom[self.tail]
+            atom2 = resB._name2atom[other.head]
+            new_molecule.Add_Residue_Link(atom1, atom2)
+            _link_residue_process_coordinate(new_molecule, atom1, atom2)            
         return new_molecule
     elif type(other) == Molecule:
         new_molecule = other.deepcopy()
@@ -238,21 +268,11 @@ def ResidueType_Add(self, other):
         for atom in self.atoms:
             resA.Add_Atom(atom)
         new_molecule.residues.insert(0, resA)
-        assert resA.type.tail and resB.type.head, "+ and * can only be used to link head and tail"
-        atom1 = resA._name2atom[resA.type.tail]
-        atom2 = resB._name2atom[resB.type.head] 
-        new_molecule.Add_Residue_Link(atom1, atom2)
-        if resA.type.tail_second:
-            atom10 = resA._name2atom[resA.type.tail_second]
-        else:
-            atom10 = None
-        if resA.type.tail_third:
-            atom100 = resA._name2atom[resA.type.tail_third]
-        else:
-            atom100 = None
-        _link_residue_process_coordinate(new_molecule, atom1, atom2, atom10, atom100, 
-            resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)   
-        
+        if resA.type.tail and resB.type.head:
+            atom1 = resA._name2atom[resA.type.tail]
+            atom2 = resB._name2atom[resB.type.head] 
+            new_molecule.Add_Residue_Link(atom1, atom2)
+            _link_residue_process_coordinate(new_molecule, atom1, atom2)   
         return new_molecule
     elif type(other) == type(None):
         return self
@@ -267,21 +287,11 @@ def Molecule_Add(self, other):
         for atom in other.atoms:
             resB.Add_Atom(atom)
         new_molecule.Add_Residue(resB)
-        assert resA.type.tail and resB.type.head, "+ and * can only be used to link head and tail"
-        atom1 = resA._name2atom[resA.type.tail]
-        atom2 = resB._name2atom[resB.type.head]
-        new_molecule.Add_Residue_Link(atom1, atom2)
-        if resA.type.tail_second:
-            atom10 = resA._name2atom[resA.type.tail_second]
-        else:
-            atom10 = None
-        if resA.type.tail_third:
-            atom100 = resA._name2atom[resA.type.tail_third]
-        else:
-            atom100 = None
-        _link_residue_process_coordinate(new_molecule, atom1, atom2, atom10, atom100, 
-            resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)    
-        
+        if resA.type.tail and resB.type.head:
+            atom1 = resA._name2atom[resA.type.tail]
+            atom2 = resB._name2atom[resB.type.head]
+            new_molecule.Add_Residue_Link(atom1, atom2)
+            _link_residue_process_coordinate(new_molecule, atom1, atom2)     
         return new_molecule
     elif type(other) == Molecule:
         new_molecule = self.deepcopy()
@@ -290,21 +300,11 @@ def Molecule_Add(self, other):
         resB = new_molecule2.residues[0]
         for res in new_molecule2.residues:
             new_molecule.Add_Residue(res)
-        assert resA.type.tail and resB.type.head, "+ and * can only be used to link head and tail"
-        atom1 = resA._name2atom[resA.type.tail]
-        atom2 = resB._name2atom[resB.type.head]
-        new_molecule.Add_Residue_Link(atom1, atom2)
-        if resA.type.tail_second:
-            atom10 = resA._name2atom[resA.type.tail_second]
-        else:
-            atom10 = None
-        if resA.type.tail_third:
-            atom100 = resA._name2atom[resA.type.tail_third]
-        else:
-            atom100 = None
-        _link_residue_process_coordinate(new_molecule, atom1, atom2, atom10, atom100, 
-            resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)   
-
+        if resA.type.tail and resB.type.head:
+            atom1 = resA._name2atom[resA.type.tail]
+            atom2 = resB._name2atom[resB.type.head]
+            new_molecule.Add_Residue_Link(atom1, atom2)
+            _link_residue_process_coordinate(new_molecule, atom1, atom2)   
         return new_molecule
     elif type(other) == type(None):
         return self
@@ -318,20 +318,11 @@ def iMolecule_Add(self, other):
         for atom in other.atoms:
             resB.Add_Atom(atom)
         self.Add_Residue(resB)
-        if resA.type.tail and other.head:
+        if resA.type.tail and resB.type.head:
             atom1 = resA._name2atom[resA.type.tail]
             atom2 = resB._name2atom[other.head]
             self.Add_Residue_Link(atom1, atom2)
-            if resA.type.tail_second:
-                atom10 = resA._name2atom[resA.type.tail_second]
-            else:
-                atom10 = None
-            if resA.type.tail_third:
-                atom100 = resA._name2atom[resA.type.tail_third]
-            else:
-                atom100 = None
-            _link_residue_process_coordinate(self, atom1, atom2, atom10, atom100, 
-                resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)  
+            _link_residue_process_coordinate(self, atom1, atom2)  
         return self
     elif type(other) == Molecule:
         new_molecule2 = other.deepcopy()
@@ -343,16 +334,7 @@ def iMolecule_Add(self, other):
             atom1 = resA._name2atom[resA.type.tail]
             atom2 = resB._name2atom[resB.type.head]
             self.Add_Residue_Link(atom1, atom2)
-            if resA.type.tail_second:
-                atom10 = resA._name2atom[resA.type.tail_second]
-            else:
-                atom10 = None
-            if resA.type.tail_third:
-                atom100 = resA._name2atom[resA.type.tail_third]
-            else:
-                atom100 = None 
-            _link_residue_process_coordinate(self, atom1, atom2, atom10, atom100, 
-                resA.type.tail_bond, resA.type.tail_angle, resA.type.tail_dihedral)   
+            _link_residue_process_coordinate(self, atom1, atom2)   
         return self
     elif type(other) == type(None):
         return self
@@ -361,17 +343,20 @@ def iMolecule_Add(self, other):
 
 def Muls(self, other):
     if type(other) == int:
-        assert other >= 0
-        t = self
+        assert other >= 1
+        if type(self) == ResidueType:
+            t = self
+        else:
+            t = self.deepcopy()
         for i in range(other - 1):
-            t = t + self
+            t += self
         return t
     else:
         raise TypeError("unsupported operand type(s) for +: '%s' and '%s'"%(type(self), type(other)))
 
 def iMuls(self, other):
     if type(other) == int:
-        assert other >= 0
+        assert other >= 1
         for i in range(other - 1):
             self += self
         return self
