@@ -186,7 +186,7 @@ def pdb(filename, judge_HIS = True):
     for count in links:
         ResA = molecule.residues[count]
         ResB = molecule.residues[count-1]
-        molecule.Add_Residue_Link(ResA._name2atom[ResA.type.connect_atoms["head"]], ResB._name2atom[ResB.type.connect_atoms["tail"]])
+        molecule.Add_Residue_Link(ResA._name2atom[ResA.type.head], ResB._name2atom[ResB.type.tail])
     return molecule
     
 sys.modules['__main__'].__dict__["loadpdb"] = pdb    
@@ -242,7 +242,7 @@ def frcmod(filename, nbtype = "RE"):
                 atoms = [ word.strip() for word in line[:11].split("-") ]
                 words = line[11:].split()
                 impropers += "-".join(atoms) + "\t" + words[0] + "\t" + words[1] + "\t" + str(int(float(words[2]))) + "\n"
-            elif flag == "NONB":
+            elif flag == "NONB" or flag == "NONBON":
                 words = line.split()
                 LJs += words[0] + "-" + words[0] + "\t" + words[1] + "\t" + words[2] + "\n"
             elif flag == "CMAP":
